#!/bin/ksh
#############################################################################
# Index Creation Phase
#############################################################################

echo START Index Creation at `date`

sqlplus / as sysdba<<EOF
connect tpch/tpch;

set echo on
set timing on

drop index i_l_orderkey;
create index i_l_orderkey
on lineitem (l_orderkey)
global partition by hash (l_orderkey)
partitions 128
pctfree 2
initrans 10
tablespace ts_idx
storage (freelist groups 4 freelists 84)
parallel 256
compute statistics
nologging ;

drop index i_o_orderkey;
create unique index i_o_orderkey
on orders (o_orderkey)
global partition by hash (o_orderkey)
partitions 128
pctfree 2
initrans 10
tablespace ts_idx  
storage (freelist groups 4 freelists 84)
parallel 256
compute statistics
nologging ;

drop index i_c_custkey;
create unique index i_c_custkey
on customer (c_custkey)
global partition by hash (c_custkey)
partitions 10
pctfree 0
initrans 10
tablespace ts_idx
storage (buffer_pool keep)
parallel 256
compute statistics
nologging ;

drop index ps_pkey_skey;
create unique index ps_pkey_skey
on partsupp (ps_partkey,ps_suppkey)
global partition by hash (ps_partkey)
partitions 256
pctfree 0
initrans 10
tablespace ts_idx
storage (initial 80m next 80m buffer_pool keep)
parallel 256
compute statistics
nologging ;


alter index i_o_orderkey allocate extent (size 10m instance 1);
alter index i_l_orderkey allocate extent (size 10m instance 1);
EOF

echo END Index Creation at `date`


