#ifdef RCSID
static char *RCSid =
   "$Header: qexecpl.c 17-oct-2001.09:29:47 mpoess Exp $ ";
#endif /* RCSID */

/* Copyright (c) 1999, 2001, Oracle Corporation.  All rights reserved.  */

/*

   NAME
     qexecpl.c - <one-line expansion of the name>

   MODIFIED   (MM/DD/YY)
      mpoess    10/17/01 - add serialization level in SQLinit
      mpoess    02/22/01 - add linux changes
      mpoess    08/05/99 - make compile
      mpoess    11/13/98 - fix pddl statement
      pswong    02/19/97 - migrating to version 8
      pswong    04/02/96 - more polishing
      pswong    03/25/96 - polish up
      pswong    03/06/96 - created

*/

#include <stdio.h>
#include <string.h>
#include <setjmp.h>
#include <sys/param.h>
#include <errno.h>
#include <math.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>
#include <stdlib.h>

#include "qexecpl.h"

extern double gettime();

int get_statement();

void sql_error();

int define_output_variables();
void process_select_list();
void usage();
void SQLinit();
void SQLexec();
void SQLexit();
void *memalloc();
void print_header();
void print_rows();
int OFEN();
void remove_newline();

char logname[UNAME_LEN];  
char *passwd;

double tr_start = 0.0;    
double tr_end = 0.0;      

double s_tr_start = 0.0;  
double s_tr_end = 0.0;    

int end_flag = 0;         

int stmt_cnt = 0;         
int qry_cnt = 0;          

double product = 1.0;     
int rows_ret = 0;      
int num_sel_list = 0;   

long num_to_fetch = -1;   

sltype slist[MAX_SEL_LIST];  
dltype *dlist[MAX_SEL_LIST]; 

char stmt[SQL_LEN];       
char qn[3];        
char qnp[3];        
char cmnt[5000];       
#ifdef LINUX
FILE *qtemp;      
FILE *logfile;   
FILE *rep;      
#else
FILE *qtemp = stdin;      
FILE *logfile = stdout;   
FILE *rep = stdout;      
#endif
void *defbuf;   
int deflen = 0;    
int deftype = 1;    

int pfmem = PFMEMSIZE;    

time_t tim;               

OCIEnv *tpcenv = NULL;
OCIServer *tpcsrv = NULL;
OCIError *errhp = NULL;
OCISvcCtx *tpcsvc = NULL;
OCISession *tpcusr = NULL;
OCIStmt *curq = NULL;
OCIStmt *cur_dml = NULL;
OCIStmt *cur_ddl = NULL;
OCIParam *tpcpar = NULL;

sword status = OCI_SUCCESS;  

void usage() {

  fprintf(stderr,"\nUsage: qexec username/password [q<path name for query template file>]\n");
  fprintf(stderr,"                 [l<path name for log>] [r<path name for reports>]\n\n");
  fprintf(stderr,"Options:\n");
  fprintf(stderr,"q<path for query>         : full path name for the query template file.\n");
  fprintf(stderr,"                            (default is stdin)\n");
  fprintf(stderr,"l<path name for log>      : full path name for log files\n");
  fprintf(stderr,"                            (default is stdout)\n");
  fprintf(stderr,"r<path name for reports>  : full path name for reports\n");
  fprintf(stderr,"                            (default is stdout)\n");
  exit(-1);
}



void sql_error(errhp,status,type)
     OCIError *errhp;
     sword status;
     sword type;
{
  char msg[2048];
  ub4 errcode;
  ub4 msglen;
  int i,j;

  switch(status) {
  case OCI_SUCCESS_WITH_INFO:
    fprintf(stderr, "Error: Statement returned with info.\n");
    if (type)
      (void) OCIErrorGet(errhp,1,NULL,(sb4*)&errcode,(text*)msg,
			 2048,OCI_HTYPE_ERROR);
    else
      (void) OCIErrorGet(errhp,1,NULL,(sb4*)&errcode,(text*)msg,
			 2048,OCI_HTYPE_ENV);
    fprintf(stderr,"%s\n",msg);
    break;
  case OCI_ERROR:
    fprintf(stderr, "Error: OCI call error.\n");
    if (type)
      (void) OCIErrorGet(errhp,1,NULL,(sb4*)&errcode,(text*)msg,
			 2048,OCI_HTYPE_ERROR);
    else
      (void) OCIErrorGet(errhp,1,NULL,(sb4*)&errcode,(text*)msg,
			 2048,OCI_HTYPE_ENV);
    fprintf(stderr,"%s\n",msg);
    break;
  case OCI_INVALID_HANDLE:
    fprintf(stderr, "Error: Invalid Handle.\n");
    if (type)
      (void) OCIErrorGet(errhp,1,NULL,(sb4*)&errcode,(text*)msg,
			 2048,OCI_HTYPE_ERROR);
    else
      (void) OCIErrorGet(errhp,1,NULL,(sb4*)&errcode,(text*)msg,
			 2048,OCI_HTYPE_ENV);
    fprintf(stderr,"%s\n",msg);
    break;
  }

  (void) OCITransRollback(tpcsvc,errhp,OCI_DEFAULT);

  fprintf(stderr, "Exiting Oracle...\n");
  fflush(stderr);
  
  SQLexit();
  
  exit(1);
}

#ifdef LINUX
int main(argc,argv)
#else
void main(argc,argv)
#endif
     int argc;
     char *argv[];
{

  int i,pos,pos2;
  int retcode;      
#ifdef LINUX
  logfile=fopen("/dev/stdout","w");
  qtemp=fopen("/dev/stdin","rw");
  rep=fopen("/dev/stdout","w");
#endif

  if ((argc > 5) || (argc < 2)) {
    usage();
  }

  strcpy(logname, argv[1]);

  argc -= 1;
  argv += 1;

  while(--argc) {
    ++argv;
    switch(argv[0][0]) {
    case 'q':
      if ((qtemp = fopen(++(argv[0]),"r")) == NULL) {
	fprintf(stderr,"Unable to open file '%s'\n", argv[0]);
	fprintf(stderr,"%s: %s\n", argv[0], strerror(errno));
	exit(-1);
      }
      break;
    case 'r':
      if ((rep = fopen(++(argv[0]),"a")) == NULL) {
	fprintf(stderr,"Unable to open file '%s'\n", argv[0]);
	fprintf(stderr,"%s: %s\n", argv[0], strerror(errno));
	exit(-1);
      }
      break;
    case 'l':
      if ((logfile = fopen(++(argv[0]),"a")) == NULL) {
	fprintf(stderr,"Unable to open file '%s'\n", argv[0]);
	fprintf(stderr,"%s: %s\n", argv[0], strerror(errno));
	exit(-1);
      }
      break;
    default:
      fprintf(stderr,"Invalid Option: %c\n", argv[0][0]);
      usage();
      break;
    }
  }

  SQLinit();

  time(&tim);
  fprintf(logfile, "Begin Execution at %s\n\n", ctime(&tim));
  fprintf(rep, "Begin Executing this Stream at %s\n\n", ctime(&tim));

  while ((retcode = get_statement()) > 0) {

    switch (retcode) {

    case COMMENT:
      /*if (end_flag) {
	end_flag = 0;  */    /* reset query end flag */
	/* save the comment so that we can print it out later on  */
	/* strcpy(cmnt, stmt);
	break;
      } */
      if (stmt[3]== '@') {
         pos=4;
	 strcpy(qnp,qn);
         while (stmt[pos] != ')') {
           pos++;
         }
         pos2=0;
         pos++;
         while (stmt[pos] != '.') {
           /*printf ("qn %d %c \n",pos2,stmt[pos]);*/
           qn[pos2]=stmt[pos];
           pos2++;
           pos++;
         }
         qn[pos2] = 0;
         /* printf("found a new query: %s\n",qn); */
      }
      strcat(cmnt, stmt);
      break;

    case SET_FETCHROW:
      fprintf(logfile,"Setting the number of rows to fetch to: %ld\n\n",
	      num_to_fetch);
      break;

    case SQL_STMT:

      SQLexec();

      stmt_cnt++;
      qry_cnt++;
      fflush(rep);
      fflush(logfile);
      /* 
      fprintf(logfile,"\nStatement Started at %.2f\n", s_tr_start);
      fprintf(logfile,"Statement Ended at %.2f\n", s_tr_end);

      fprintf(logfile,"Statement Processed in %.2f seconds.\n",
              (s_tr_end - s_tr_start));
      fprintf(rep, "Query %s: Execution Time: %.2f started %.2f ended %.2f\n",
                    qn,(s_tr_end - s_tr_start)s_tr_start,s_tr_end);
      fflush(rep);
      fflush(logfile);*/
      break;

    default:
      fprintf(stderr, "Invalid statement type!!\n");
      SQLexit();
      break;
    }
  }
  
  tr_end = gettime();

  fprintf(logfile,"Query Processed in %.2f seconds.\n\n",(tr_end - s_tr_start));

  /* fprintf(rep, "Query %s : Execution time %.2f\n", qn,(tr_end - s_tr_start));*/
  fprintf(rep, "Query %s: Execution Time: %.2f started %.2f ended %.2f\n",
                    qn,(tr_end - s_tr_start),s_tr_start,tr_end);

  time(&tim);
  fprintf(logfile,"\nEnded Executing this Stream at %s\n", ctime(&tim));
  fprintf(logfile,"\nStream Started at %.2f\n", tr_start);
  fprintf(logfile,"Stream Ended at %.2f\n", tr_end);
  fprintf(logfile,"Stream Processed in %.2f seconds\n\n", (tr_end - tr_start));

  fprintf(rep,"\nEnded Executing this Stream at %s\n", ctime(&tim));
  fprintf(rep,"\nStream Started at %.2f\n", tr_start);
  fprintf(rep,"Stream Ended at %.2f\n", tr_end);
  fprintf(rep,"Stream Processed in %.2f seconds\n\n", (tr_end - tr_start));
   
  fprintf(logfile, "\nSQL statements processed: %d\n", stmt_cnt);
  /*fprintf(logfile, "Queries processed: %d\n", qry_cnt);*/

  fflush(rep);
  fflush(logfile);

  fclose(qtemp);

  SQLexit();
  exit(0);
}

void SQLinit() {

  int i;

  for (i=0; i<MAX_SEL_LIST; i++) {
    if (i < MAX_PREALLOC) {
      dlist[i] = (dltype *) memalloc (sizeof(dltype));
      dlist[i]->defhdl = NULL;
/*      OCIhalloc(curq,&(dlist[i]->defhdl),OCI_HTYPE_DEFINE); */
    }
    else
      dlist[i] = NULL;
  }

  (void) OCIInitialize(OCI_DEFAULT,(dvoid *)0,0,0,0);

  if((status=OCIEnvInit((OCIEnv **)&tpcenv,OCI_DEFAULT,0,(dvoid **)0)) !=
     OCI_SUCCESS)
    sql_error(tpcenv, status, 0);

  OCIhalloc(tpcenv,&errhp,OCI_HTYPE_ERROR);
  OCIhalloc(tpcenv,&curq,OCI_HTYPE_STMT);
  OCIhalloc(tpcenv,&cur_dml,OCI_HTYPE_STMT);
  OCIhalloc(tpcenv,&cur_ddl,OCI_HTYPE_STMT);
  OCIhalloc(tpcenv,&tpcsvc,OCI_HTYPE_SVCCTX);
  OCIhalloc(tpcenv,&tpcsrv,OCI_HTYPE_SERVER);
  OCIhalloc(tpcenv,&tpcusr,OCI_HTYPE_SESSION);

  passwd = strchr(logname, '/');
  *passwd = '\0';
  passwd++;

  if ((status = OCIServerAttach(tpcsrv,errhp,(text *)0,0,OCI_DEFAULT)) != OCI_SUCCESS) 
    sql_error(errhp,status,1);

  OCIaset(tpcsvc,OCI_HTYPE_SVCCTX,tpcsrv,0,OCI_ATTR_SERVER,errhp);
  OCIaset(tpcusr,OCI_HTYPE_SESSION,logname,strlen(logname),OCI_ATTR_USERNAME,
	  errhp);
  OCIaset(tpcusr,OCI_HTYPE_SESSION,passwd,strlen(passwd),OCI_ATTR_PASSWORD, 
	  errhp);

  if ((status = OCISessionBegin(tpcsvc, errhp, tpcusr, OCI_CRED_RDBMS, 
				OCI_DEFAULT)) != OCI_SUCCESS)
    sql_error(errhp,status,1);
 
  OCIaset(tpcsvc,OCI_HTYPE_SVCCTX,tpcusr,0,OCI_ATTR_SESSION,errhp);


/*
  if ((status=OCILogon((OCIEnv *)tpcenv,(OCIError *)errhp,(OCISvcCtx *)tpcsvc,
		  (text *)logname, strlen(logname), (text *)passwd, 
		  strlen(passwd), (text *) 0, 0)) != OCI_SUCCESS)
    sql_error(errhp, status, 1);
*/
  printf("\nConnected to ORACLE as user: %s\n\n", logname);

}

void SQLexec() 
{
  int i;
  ub2 stmttyp = OCI_STMT_SELECT;     /* default is a SELECT statement */

  if (qry_cnt) {
     time(&tim);
     s_tr_end = gettime();
     fprintf(logfile,"Query Processed in %.2f seconds.\n\n", 
	    (s_tr_end - s_tr_start));

     /*fprintf(rep, "Query %s : Execution time %.2f\n", qnp,(s_tr_end - s_tr_start));*/
     fprintf(rep, "Query %s: Execution Time: %.2f started %.2f ended %.2f\n",
                    qnp,(s_tr_end - s_tr_start),s_tr_start,s_tr_end);

      /* Let's fflush stuff so that we can see what's going on */

      fflush(logfile);
      fflush(rep);
    }
  else
     tr_start = gettime();

  s_tr_start = gettime();

  if ((status = OCIStmtPrepare(curq, errhp, (text*) stmt, (ub4) strlen(stmt), 
			       OCI_NTV_SYNTAX, OCI_DEFAULT)) != OCI_SUCCESS)
    sql_error(errhp,status,1);

  fprintf(logfile, "\n%s\n", cmnt);
  cmnt[0]=0;
  fprintf(logfile, "\n%s\n", stmt);
  
  OCIaget(curq,OCI_HTYPE_STMT,&stmttyp,NULL,OCI_ATTR_STMT_TYPE,errhp);

  if (stmttyp != OCI_STMT_SELECT) {
    OCIsexec(tpcsvc,curq,errhp,1);
    return;
  } 

  OCIaset(curq, OCI_HTYPE_STMT, &pfmem, 0, OCI_ATTR_PREFETCH_MEMORY, errhp);

  OCIsexec(tpcsvc,curq,errhp,0);

  num_sel_list = define_output_variables();
    
  (void) process_select_list(num_sel_list);

  OCIaget(curq,OCI_HTYPE_STMT,&rows_ret,NULL,OCI_ATTR_ROW_COUNT,errhp);

  i=MAX_PREALLOC;
  while(dlist[i] != NULL) {
    free(dlist[i]);
    dlist[i++] = NULL;
  }

  num_to_fetch = -1;

}


void SQLexit() {

  int i;

  OCILogoff(tpcsvc,errhp);
  OCIhfree(tpcenv,OCI_HTYPE_STMT);
  OCIhfree(tpcsvc,OCI_HTYPE_SVCCTX);
  OCIhfree(tpcsrv,OCI_HTYPE_SERVER);
  OCIhfree(tpcusr,OCI_HTYPE_SESSION);

  for (i=0; i<MAX_SEL_LIST; i++) {
    if (dlist[i] != NULL) {
      free(dlist[i]);
    }
  }

  fflush(rep);
  fflush(logfile);

}


int define_output_variables()
{

  int i;
  int retflag = 0;

  for (i=0; i<MAX_SEL_LIST; i++) {
    
    slist[i].buflen = MAX_COLNAME_SIZE;

    if (OCIParamGet(curq, OCI_HTYPE_STMT, errhp, (dvoid **) &tpcpar,
			       POS(i)) != OCI_SUCCESS)
      break;
	

    OCIaget(tpcpar, OCI_DTYPE_PARAM, &(slist[i].dbsize),
	    NULL, OCI_ATTR_DATA_SIZE, errhp);
    OCIaget(tpcpar, OCI_DTYPE_PARAM, &(slist[i].dbtype),
	    NULL, OCI_ATTR_DATA_TYPE, errhp);
    OCIaget(tpcpar, OCI_DTYPE_PARAM, &(slist[i].buf),
	    &(slist[i].buflen), OCI_ATTR_NAME, errhp);
    OCIaget(tpcpar, OCI_DTYPE_PARAM, &(slist[i].precision),
	    NULL, OCI_ATTR_PRECISION, errhp);
    OCIaget(tpcpar, OCI_DTYPE_PARAM, &(slist[i].scale),
	    NULL, OCI_ATTR_SCALE, errhp);
	    
/*
    if (slist[i].buflen < MAX_COLNAME_SIZE) 
      (slist[i].buf)[slist[i].buflen] = '\0';
*/

    if (i >= MAX_PREALLOC) {
      dlist[i] = (dltype *) memalloc(sizeof(dltype));
      dlist[i]->defhdl = NULL;
    }

    switch (slist[i].dbtype) {
      
    case OCI_TYPECODE_NUMBER:

#ifdef HAVE_SCALE
      if (slist[i].scale != 0) {
	defbuf = (double *) dlist[i]->fbuf;
	deflen = FLT;
	deftype = OCI_TYPECODE_DOUBLE;
	slist[i].dbtype = OCI_TYPECODE_DOUBLE;
      } else {
	defbuf = (int *) dlist[i]->ibuf;
	deflen = INT;
	deftype = OCI_TYPECODE_INTEGER;
	slist[i].dbtype = OCI_TYPECODE_INTEGER;
      }
#else
      defbuf = (double *) dlist[i]->fbuf;
      deflen = FLT;
      deftype = OCI_TYPECODE_FLOAT;
      slist[i].dbtype = OCI_TYPECODE_FLOAT;
#endif /* HAVE_SCALE */

      break;
    
    default:
      
      /* default is character string */
      
      defbuf = (char **) dlist[i]->sbuf;
      deflen = MAX_STR_LEN;
      deftype = SQLT_STR;
/*      deftype = OCI_TYPECODE_CHAR; */
      break;
    }

    if ((status=OCIDefineByPos(curq,&(dlist[i]->defhdl),errhp,POS(i),
			       defbuf,deflen,deftype,NULL,
			       dlist[i]->rlen,NULL,OCI_DEFAULT))!=OCI_SUCCESS) 
      sql_error(errhp,status,1);
  }
  return i;
}


void process_select_list(num)
     int num;        /* number of select list items */
{

  int i,j;
  int ntf;
  int num_so_far;
  sword stats = OCI_SUCCESS;

  print_header(num);

  ntf = (num_to_fetch >= 0) ? num_to_fetch : MAX_ARRAY;

  if ((ntf > MAX_ARRAY) || (num_to_fetch == -1)) {

    stats = OCIStmtFetch(curq, errhp, MAX_ARRAY, OCI_FETCH_NEXT, OCI_DEFAULT);

    OCIaget(curq,OCI_HTYPE_STMT,&rows_ret,NULL,OCI_ATTR_ROW_COUNT,errhp);

    print_rows(num,rows_ret);

    if (stats != OCI_NO_DATA) {
      if (num_to_fetch == -1) {
	while ((stats = OCIStmtFetch(curq,errhp,MAX_ARRAY,OCI_FETCH_NEXT,
				     OCI_DEFAULT)) == OCI_SUCCESS) {
	  OCIaget(curq,OCI_HTYPE_STMT,&num_so_far,NULL,
		     OCI_ATTR_ROW_COUNT,errhp);
	  print_rows(num,(num_so_far-rows_ret));
	  rows_ret = num_so_far;
	}
	OCIaget(curq,OCI_HTYPE_STMT,&num_so_far,NULL,
		   OCI_ATTR_ROW_COUNT,errhp);
	print_rows(num,(num_so_far-rows_ret));
	rows_ret = num_so_far;
      } else {
	ntf -= MAX_ARRAY;

	while ((stats = OCIStmtFetch(curq,errhp,
				      ((ntf>MAX_ARRAY) ? MAX_ARRAY:ntf),
				      OCI_FETCH_NEXT, OCI_DEFAULT)) ==
	       OCI_SUCCESS) {
	  ntf -= MAX_ARRAY;
	  OCIaget(curq,OCI_HTYPE_STMT,&num_so_far,NULL,
		     OCI_ATTR_ROW_COUNT,errhp);
	  print_rows(num,(num_so_far-rows_ret));
	  rows_ret = num_so_far;
	  if (ntf <= 0) break;
	}
	OCIaget(curq,OCI_HTYPE_STMT,&num_so_far,NULL,
		   OCI_ATTR_ROW_COUNT,errhp);
	print_rows(num,(num_so_far-rows_ret));
	rows_ret = num_so_far;
      }
    }
  } else {
    OCIStmtFetch(curq, errhp, ntf, OCI_FETCH_NEXT, OCI_DEFAULT);
    OCIaget(curq,OCI_HTYPE_STMT,&rows_ret,NULL,OCI_ATTR_ROW_COUNT,errhp);
    print_rows(num,rows_ret);
  }

  fprintf(logfile,"\n\n%d row%c processed.\n", rows_ret, 
	  rows_ret == 1 ? '\0' : 's');

}


int get_statement()
{

  char line[128];
  char *pos, *str;

  stmt[0] = '\0';

  while (fgets(line, 127, qtemp) != NULL) {

    if (line[0] == '\n')
      continue;

    str = line;

    while (*str == ' ') str++;

    strcat(stmt, str);

    if ((str[0] == '-') && (str[1] == '-'))
      return COMMENT;

    if (strncmp(str, "set_fetchrows", 13) == 0) {
      pos = strchr(str, ';');
      *pos = '\0';
      pos = strchr(str, '=');
      num_to_fetch = atol(++pos);
      return SET_FETCHROW;
    }

    if ((pos = strchr(stmt, ';')) != NULL) {
      *pos = '\0';
      return SQL_STMT;
    }
  }
  return END_OF_FILE;
}   


void *memalloc(size)
     int size;
{

  void *tmp;

  if ((tmp = (void *) malloc(size)) == NULL) {
    fprintf(stderr, "Error in malloc\n");
    SQLexit();
    return NULL;       /* should never reach here */
  } else {
    return tmp;
  }
}



void print_header(nsel)
     int nsel;   
{

  int i, diff;
  char colname[MAX_COLNAME_SIZE];
  int len = 0;   
  int cwid = 0;

  fprintf(logfile, "\n");
  
  for (i=0; i<nsel; i++) {

    strncpy((char *)colname, (char *)slist[i].buf, slist[i].buflen);
    colname[slist[i].buflen] = '\0';

    cwid = MAX(slist[i].dbsize, slist[i].buflen);

    if (cwid > 80) {
      fprintf(logfile,"\n");
      len = 0;
    } else if ((len += cwid) > 80) {
      fprintf(logfile,"\n");
      len = cwid;
    }
#ifdef FORMAT1
    if ((slist[i].dbtype == INT_TYPE) || (slist[i].dbtype == FLT_TYPE))
      fprintf(logfile, "%*s ", cwid, slist[i].buf);
    else  /* string type */
      fprintf(logfile, "%*s ", -cwid, slist[i].buf);
#else
      fprintf(logfile, "%*s ", -cwid, colname);
#endif /* FORMAT1 */
  }

  fprintf(logfile,"\n");
}


void print_rows(ncol, nrow)
     int ncol;
     int nrow;
{

  int i,j;
  int len;
  int diff;
  int cwid;

  for (i=0;i<nrow;i++) {

    len = 0;

    for (j=0;j<ncol;j++) {
      
      cwid = MAX(slist[j].dbsize, slist[j].buflen);

      if (cwid > 80) {
	fprintf(logfile,"\n");
	len = 0;
      } else if ((len += cwid) > 80) {
	fprintf(logfile,"\n");
	len = cwid;
      }

      switch(slist[j].dbtype) {
      case INT_TYPE:
#ifdef HAVE_SCALE
	fprintf(logfile, "%*ld|", cwid,	(dlist[j]->ibuf)[i]);
	break;
#endif /* HAVE_SCALE */
      case FLT_TYPE:
#ifdef FORMAT1
	fprintf(logfile,"%*.2f ", cwid, (dlist[j]->fbuf)[i]);
#else
	fprintf(logfile,"%*.2f ", -cwid, (dlist[j]->fbuf)[i]);
#endif /* FORMAT1 */
	break;
      default:
	fprintf(logfile, "%*s ", -(cwid), (dlist[j]->sbuf)[i]);	
	break;
      }
    }
    fprintf(logfile, "\n");
  }
}


void remove_newline(str)
     char *str;
{

  char *p;

  while ((p = strchr(str,'\n')) != NULL)
    *p = ' ';
}
