#!/bin/ksh
#
# $Header: runuf2.sh 25-oct -2001.15:56:05 mpoess Exp $
#
# runuf2.sh
#
# Copyright (c) 1999, 2001, Oracle Corporation. All rights reserved.
#
# NAME
# runuf2.sh - <one-line expansion of the name>
#
# DESCRIPTION
# runuf2.sh [-u <uid/passwd to login>] [-p <program>] <run_id>
# <scale factor> <pair number> <parallelism>
# USAGE
# To execute UF2.
#
# NOTES
# <other useful comments, qualifications, etc.>
#
#
. $KIT_DIR/env
UPDATE_DIR=${KIT_DIR}/update
SCRIPT_DIR=${UPDATE_DIR}/scripts
UTILS_DIR=${KIT_DIR}/utils
GTIME=${SCRIPT_DIR}/gtime
LOG_DIR=${UPDATE_DIR}/log
PAR_HINT=96
SF=${SCALE_FACTOR}
PASSWD=${DATABASE_USER}


START=`$GTIME`

if [ $# -lt 1 ]
then
usage
exit 1
fi
SETNUM=$1
i=1
PID=""


sqlplus /NOLOG << !
connect $PASSWD;
set timing on
set serveroutput on
set echo on

alter session set parallel_degree_policy='MANUAL';


drop directory data_dir;
create directory data_dir as '/ff_updates/3TB';
drop table temp_okey_et;
drop table temp_okey;

create table temp_okey_et (
    t_orderkey number
)
organization external (
type ORACLE_LOADER
default directory data_dir
access parameters
(
	records delimited by newline
        nobadfile 
        nologfile 
	fields terminated by '|'
	missing field values are null
    )
    location (
    'delete.${SETNUM}'))
reject limit unlimited parallel ${PAR_HINT};

create table temp_okey parallel ${PAR_HINT} nologging as select * from temp_okey_et;
create unique index i_temp_okey on temp_okey (t_orderkey) parallel ${PAR_HINT}  nologging compute statistics;

alter session force parallel dml parallel ${PAR_HINT};
alter session set isolation_level=serializable;
alter session set optimizer_index_cost_adj=25;

delete from (select /*+ use_nl(t o) leading(t) */ o.rowid from orders o, temp_okey t
    where o.o_orderkey = t.t_orderkey order by 1);

delete from (select /*+ use_nl(t l) leading(t) */ l.rowid from lineitem l,temp_okey t
    where l.l_orderkey = t.t_orderkey order by 1);

commit;

drop table temp_okey;
drop table temp_okey_et;
exit;
!

END=`$GTIME`


echo ""
echo "Update Function 2 Set $SETNUM done!"
echo "Elapsed Time is `echo $END - $START | bc`"
echo ""
